package com.incubator.game.contest;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.DateUtils;
import com.incubator.common.util.StringUtils;
import com.incubator.game.GGame;
import com.incubator.game.data.data.GameData;
import com.incubator.game.data.data.PromotionLvCO;
import com.incubator.game.data.data.ext.ContestExt;
import com.incubator.game.data.mq.RabbitMQUtil;
import com.incubator.game.data.po.ContestPO;
import com.incubator.game.handler.contest.GetContestListHandler;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.Player;
import com.incubator.game.room.GDRoom;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.room.ZDRoom;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

/**
 * 官方比赛实体类
 */
public class Contest {

    /**
     * 比赛数据
     **/
    public ContestPO data;
    /**
     * 比赛策划表配置
     **/
    public ContestExt prop;
    /**
     * 当前晋级配置
     **/
    public PromotionLvCO propPromotionLv;

    /**
     * 激活标记
     **/
    private boolean isActive;
    /**
     * 检测标记
     **/
    public boolean flag;
    /**
     * 时间秒数
     **/
    public int time;
    /**
     * 比赛状态(0等待/报名中 1比赛开始 2进行中 3已结束)
     **/
    public int state;

    /**
     * 比赛当前状态0等待/报名中 1比赛开始 2进行中 3当局结束
     */
    public int contestState;

    /**
     * 每个房间玩家容量
     **/
    private final int roomCapacity = 4;

    /**
     * 比赛报名角色 [key:玩家id, value:玩家对象]
     **/
    public Map<String, Player> actors = new ConcurrentHashMap<>();
    /**
     * 比赛房间列表
     **/
    public Map<Long, Room> rooms = new ConcurrentHashMap<>();
    /**
     * 等待队列
     **/
    private final ConcurrentLinkedQueue<String> waitingPlayers = new ConcurrentLinkedQueue<>();
    /**
     * 淘汰队列
     */
    private final ConcurrentLinkedQueue<String> DieOutPlayers = new ConcurrentLinkedQueue<>();
    /**
     * 替补队列
     */
    private final ConcurrentLinkedQueue<String> SubstitutePlayers = new ConcurrentLinkedQueue<>();

    /**
     * 是否进行下一轮
     **/
    public boolean isNext = true;

    /**
     * 比赛任务主线程
     **/
    private ScheduledExecutorService scheduler;

    public Contest() {
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    /**
     * 初始化
     *
     * @param roomType  比赛玩法类型
     * @param contestId 比赛id
     * @param prop      比赛策划表配置
     */
    public void init(int roomType, long contestId, ContestExt prop) {
        this.prop = prop;
        this.prop.competeType = prop.competeType;
        // 初始化默认第一条配置
        //todo 测试 4人房 最低2人
//        this.propPromotionLv = GameData.PromotionLvs.get(1);
        this.propPromotionLv = new PromotionLvCO();
        this.propPromotionLv.iD = contestId;
        this.propPromotionLv.max = prop.getMax();
        this.propPromotionLv.min = prop.getMin();
        // 初始化比赛
        this.data = new ContestPO();
        this.data.contestId = contestId;
        // todo 测试
        this.data.startTime = LocalDateTime.parse(this.prop.openTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).atZone(ZoneId.of("Asia/Shanghai")).toInstant().toEpochMilli();
        Log.info("比赛开始时间:{}", this.data.startTime);
        this.data.level = this.prop.level;
    }

    /**
     * 根据当前人数刷新晋级配置
     *
     * @param num 实际报名人数
     */
    public void refreshPromotionLv(int num) {
//        for (PromotionLvCO config : GameData.PromotionLvs.values()) {
        if (num >= this.propPromotionLv.min && num <= this.propPromotionLv.max) {
            this.propPromotionLv.num = num / 2;
        }
//        }
    }

    /**
     * 玩家报名加入比赛
     *
     * @param player
     */
    public synchronized void joinContest(Player player) {
        // 加入比赛角色缓存
        if (!this.actors.containsKey(player.getId())) {
            this.actors.put(player.getId(), player);
        }
        if (!this.data.playerList.contains(player.getId())) {
            this.data.playerList.add(player.getId());
        }
        // 加入等待队列
        if (!this.waitingPlayers.contains(player.getId())) {
            this.waitingPlayers.add(player.getId());
        }
        // 根据报名人数,刷新晋级配置
        refreshPromotionLv(this.actors.size());
        if (this.propPromotionLv == null) {
            Log.error("根据报名人数刷新晋级配置出错...比赛 : {}, {}, 人数 : {}",
                    this.data.contestId, this.prop.desc, this.actors.size());
        }
        //给其他人推送当前
        GGame.onlinePlayers.forEach((id, p) -> {
            Map<String, Object> map = ProtoUtil.contestInfoToMessage(this);
            p.receive(CommonProto.Cmd.GetSignUpNum_VALUE, map);
        });
    }

    /**
     * 玩家报名加入比赛
     *
     * @param player
     */
    public synchronized void quitContest(Player player) {
        this.actors.remove(player.getId());
        this.data.playerList.remove(player.getId());
        this.waitingPlayers.remove(player.getId());
        // 根据报名人数,刷新晋级配置
        refreshPromotionLv(this.actors.size());
        if (this.propPromotionLv == null) {
            Log.error("根据报名人数刷新晋级配置出错...比赛 : {}, {}, 人数 : {}",
                    this.data.contestId, this.prop.desc, this.actors.size());
        }
        //给其他人推送当前
        this.actors.forEach((id, p) -> {
            Map<String, Object> map = ProtoUtil.contestInfoToMessage(this);
            p.receive(CommonProto.Cmd.GetSignUpNum_VALUE, map);
        });
    }

    /**
     * 加入候补
     */
    public synchronized void candidateContest(Player player) {
        // 加入候补队列
        if (!this.SubstitutePlayers.contains(player.getId())) {
            this.SubstitutePlayers.add(player.getId());
        }
    }

    /**
     * 开启官方比赛主线程
     */
    public void startContestTask() {
        this.isActive = this.prop.getRunStatus().equals("Y");
        this.scheduler.scheduleAtFixedRate(() -> {
            try {
                if (this.isActive) {
                    this.doUpdate();
                }
            } catch (Exception e) {
                this.isActive = false;
                this.state = 3; //已结束
                e.printStackTrace();
                Log.error("官方比赛执行出错...{}", e);
                //移除所有玩家
                this.actors.forEach((playerId, player1) -> {
                    player1.data.contestId = 0;
                    player1.data.roomId = 0;
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("message", "当前比赛异常结束,参加的费用将会原路返回");
                    player1.receive(CommonProto.Cmd.EndOfRace_VALUE, data);
                });
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 比赛主线程逻辑
     */
    public void doUpdate() {
        switch (this.contestState) {
            case 0:
                // 等待开始
                if (this.checkReadyStart()) {
                    this.state = 1;
                    this.contestState = 1;
                    this.flag = false;
                    this.time = 0;
                }
                break;
            case 1:
                // 比赛开始
                if (!this.flag) {
                    //开始
                    if (this.time >= 3) {
                        Log.debug("检测: 比赛id:{}, 比赛人数:{}, 状态:比赛开始...", this.data.contestId, this.waitingPlayers.size());
                        this.state = 2;
                        this.contestState = 2;
                        this.doStart();
                        this.flag = false;
                        this.time = 0;
                    }
                }
                break;
            case 2:
                // 进行中
                if (!this.flag) {
                    // 检测当轮是否结束
                    if (this.checkRoomsAllOver()) {
                        break;
                    }
                }
                break;
            case 3:
                // 下一轮
                if (this.isNext && this.time >= 10) {
//                    this.state = 1;
                    this.contestState = 1;
                    this.flag = false;
                    this.time = 0;
                }
                break;
        }

        this.time += 1;
    }

    /**
     * 检测是否可以开始 默认3秒后比赛开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        Date now = new Date();
//        if (!this.isStart(now) || this.state != 0) {
        if (!this.isStart(now) || this.contestState != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 1);
                data.put("contestTime", this.data.startTime);
                Log.debug("比赛准备开始 : 比赛 : {}, 开始时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                tmPlayer.receive(CommonProto.Cmd.MatchPreparationStart_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 比赛是否可开始
     *
     * @return
     */
    public boolean isStart(Date date) {
        // 时间是否到了
        return date.getTime() >= this.data.startTime - (3 * 1000);
    }

    /**
     * 根据玩家实时人数，动态分配比赛桌子
     */
    public synchronized void allocateRooms() {
        while (this.waitingPlayers.size() >= roomCapacity) {
            // 玩家匹配,分配房间
            JSONObject jsonData = new JSONObject();
//            jsonData.put("playingTypeOne", 2);
//            jsonData.put("playingTypeTwo", 1);
//            jsonData.put("upgradeType", 1);
//            jsonData.put("flippingType", 2);
//            jsonData.put("advanceType", 2);
//            jsonData.put("timeoutType", 1);
//            jsonData.put("voiceType", 2);
//            jsonData.put("recordType", 2);
//            jsonData.put("watchType", 4);
            cn.hutool.json.JSONObject rule = this.prop.rule;
            jsonData.put("upgradeType", rule.get("upgradeType"));
            jsonData.put("flippingType", rule.get("flippingType"));
            jsonData.put("advanceType", rule.get("advanceType"));
            jsonData.put("timeoutType", rule.get("timeoutType"));
            jsonData.put("voiceType", rule.get("voiceType"));
            jsonData.put("recordType", rule.get("recordType"));
            jsonData.put("watchType", rule.get("watchType"));
            jsonData.put("tc", rule.get("levelType")==null?0:rule.get("levelType"));
            jsonData.put("pv", rule.get("pointsType")==null?0:rule.get("pointsType"));
            jsonData.put("dv", rule.get("doublingType")==null?0:rule.get("doublingType"));
            // 房间id
            jsonData.put("roomId", RoomService.getInstance().randomRoomIdByContest());
            // 人数模式 2:2人 4:4人
            jsonData.put("playerNum", 4);
            Integer playingTypeOne = (Integer)rule.get("playingTypeOne");
            Integer playingTypeTwo = (Integer)rule.get("playingTypeTwo");
            // 玩法1 1:连续打，2:按局数，3:按时间
            jsonData.put("mode", playingTypeOne);
            if (playingTypeOne==2){ //按局数
                // 房间最大局数 分为预赛和决赛
                if (this.waitingPlayers.size() <= 4) {//决赛
                    this.prop.maxRound = this.prop.finalNum;
                } else {
                    this.prop.maxRound = this.prop.readyNum;
                }
                jsonData.put("modev", this.prop.maxRound);
            }else {//按时间
                //	 * 4:4局 6:6局 8:8局 10:10局
                //	 * 20:20分钟 30:30分钟 40:40分钟 60:60分钟
                jsonData.put("modev", playingTypeTwo);
            }
            // 房间类型 0:好友房 1:官方比赛房
            jsonData.put("type", 1);
            // 房间玩法类型 1:掼蛋
//            jsonData.put("gameType", 1);
            jsonData.put("gameType", rule.get("gameType"));

            jsonData.put("isTribute", false);

            Room room = null;
            try {
                room = RoomService.getInstance().creatRoom(null, 2, jsonData);
            } catch (Exception e) {
                Log.error("比赛开始...创建房间出错...", e);
            }
            if (room == null) {
                Log.error("比赛开始...创建房间出错...比赛 : {}, name : {}", this.data.contestId, this.prop.desc);
                return;
            }

            // 开启房间主线程
//            room.startRoomTask();

            // 添加房间缓存
            this.rooms.put(room.data.roomId, room);

            Log.info("比赛开始...比赛 : {}, name : {}, Room created: id={}, data={}",
                    this.data.contestId, this.prop.desc, room.data.roomId, room.jsonData);

            // 推送广播,拉玩家进房间
            for (int i = 0; i < this.roomCapacity; i++) {
                String playerId = this.waitingPlayers.poll();
                if (!StringUtils.isEmpty(playerId)) {
                    Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(playerId);
                    if (tmPlayer == null) {
                        //判断替补队列是否有替补玩家
                        ArrayList<String> objects = new ArrayList<>(this.SubstitutePlayers);
                        if (objects.size() > 0) {
                            for (String object : objects) {
                                tmPlayer = (Player) PlayerUtil.getOnlinePlayer(object);
                                if (tmPlayer != null) {
                                    break;
                                }
                            }
                        }
                        // todo 此处首轮需要替补机器人
                        continue;
                    }
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("roomId", room.data.roomId);
                    data.put("roomType", room.data.type);
                    data.put("type", 1);
                    Log.debug("比赛开始 : 比赛 : {}, 玩家 : {}", this.data.contestId, this.actors);
                    tmPlayer.receive(CommonProto.Cmd.MatchOfficialStart_VALUE, data);
                }
            }
        }

        // 比赛房间缓存列表
//        Map<Long, List<String>> tmpRooms = this.data.rooms.getOrDefault(this.data.curRound, new HashMap<>());
//        this.data.rooms.put(this.data.curRound, tmpRooms);
    }

    /**
     * 比赛开始
     */
    public void doStart() {
        cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
        object.put("type", "updateCompete");
        // 设置比赛数据
        this.data.curRound += 1;
        // 检测比赛人数是否满足最小人数配置
        if (this.data.curRound == 1 && this.actors.size() < this.propPromotionLv.min) {
            // 流局结束
            for (Player tmPlayer : this.actors.values()) {
                if (tmPlayer != null) {
                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("contestState", 3);
                    data.put("contestTime", new Date());
                    Log.debug("比赛结束 : 比赛 : {}, 结束时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                    tmPlayer.receive(CommonProto.Cmd.FlowOffice_VALUE, data);
                    tmPlayer.data.contestId = 0;
                    tmPlayer.data.matchPoints = 0;
                }
            }
            this.state = 3;
            this.isActive = false;
            object.put("status", "5");
            object.put("competeId", this.data.contestId);
            try {
                RabbitMQUtil.sendMessage(object.toString());
            } catch (Exception e) {
                e.printStackTrace();
            }
            return;
        }
        object.put("status", "3");
        object.put("num", this.actors.size());
        object.put("competeId", this.data.contestId);
        try {
            RabbitMQUtil.sendMessage(object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        int i = this.waitingPlayers.size() % 4;
        if (i != 0) {
            List<String> list = new ArrayList<>(this.waitingPlayers);
            // 打乱列表顺序
            Collections.shuffle(list);
            // 选择前 i 个元素
            List<String> poppedPlayers = list.subList(0, i);
            // 从原队列中移除这些元素
            for (String player : poppedPlayers) {
                this.waitingPlayers.remove(player);
                this.DieOutPlayers.add(player);
                Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(player);
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 3);
                data.put("contestTime", new Date());
                data.put("message", "您当前已被替补");
                tmPlayer.receive(CommonProto.Cmd.DieOutPlayer_VALUE, data);
            }
        }
        // 动态分配比赛房间
        this.allocateRooms();
        Log.info("当前轮次 :{}", this.data.curRound);
    }

    /**
     * 检测桌子是否全部结束
     */
    public boolean checkRoomsAllOver() {
        int cnt = 0;
        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
//            switch (room.data.gameType) {
//                case 1:
            if (room.data.curRound >= this.prop.maxRound && room.state == 3) {
                cnt += 1;
            }
//                    break;
//            }
        }

        if (cnt == this.rooms.size()) {
            this.flag = true;
            //统计分数 晋级
            this.promotionLv();
            this.propPromotionLv.num = this.rooms.size() / 2;
            if (this.rooms.size() > 1) {
                // 结束
                this.gameOver();
                //不结算 ,下一轮
                this.isNext = true;
                //清理所有房间数据
                this.rooms.clear();
                //加入替补玩家
                if (this.prop.substituteRule == 1) {
                    String playerId = this.DieOutPlayers.poll();
                    if (!StringUtils.isEmpty(playerId)) {
                        Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(playerId);
                        Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                        data.put("message", "当前比赛可替补,是否花费" + this.prop.substituteCost + "钻石替补");
                        tmPlayer.receive(CommonProto.Cmd.CandidatePlayer_VALUE, data);
                    }
                }
            } else {
                this.contestState = 3;
                this.state = 3;
                //结束比赛
                this.isActive = false;
                this.actors.forEach((playerId, player1) -> {
                    player1.data.contestId = 0;
                    player1.data.roomId = 0;
                    player1.data.matchPoints = 0;
                });
                Log.info("比赛结束 : 比赛 : {}, 结束时间 : {}", this.data.contestId, DateUtils.formatFullDate(new Date()));
                //给在线玩家发送比赛列表
                Map<String, GPlayer> onlinePlayers = GGame.onlinePlayers;
                for (GPlayer player : onlinePlayers.values()) {
                    Map<String, Object> map = new HashMap<>();
                    // 官方比赛信息列表
                    map.put("contestInfo", GetContestListHandler.contestInfoList((Player) player));
                    player.receive(CommonProto.Cmd.GetContestListRes_VALUE, map);
                }
                //发送比赛结束消息
                cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                object.put("type", "updateCompete");
                object.put("status", "4");
                object.put("competeId", this.data.contestId);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            return true;
        }
        return false;
    }

    /**
     * 统计分数 晋级
     */
    private void promotionLv() {
        Map<Long, List<Struct.PromotionData>> roomMap = this.data.promotionMap.getOrDefault(this.data.curRound, new HashMap<>());

        List<Player> objects = new ArrayList<>();
        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
            List<Struct.PromotionData> list = roomMap.getOrDefault(room.data.roomId, new ArrayList<>());
            if (room.data.result.size() >= room.data.maxNum) {
                //计算计分 头游
                Player Player1 = this.actors.getOrDefault(room.data.result.get(1).playerId, null);
                // 分
                int scores = 0;

                // 头游搭档数据
                Struct.TeammateData teammateData = ((ZDRoom) room).getTeammate(Player1.data.teammateId);
                switch (teammateData.getRank()) {
                    case 2:
                        // 搭档2游, +4
                        scores = 4;
                        break;
                    case 3:
                        // 搭档3游, +2
                        scores = 2;
                        break;
                    case 4:
                        // 搭档4游, +1
                        scores = 1;
                        break;
                }
                list.add(new Struct.PromotionData(Player1.getId(), Player1.data.avatarUrl, Player1.data.name, scores, 1, this.rooms.size() == 1));
                Player Player2 = teammateData.player;
                Player1.data.matchPoints = Player1.data.matchPoints + scores;
                Player2.data.matchPoints = Player2.data.matchPoints + scores;
                list.add(new Struct.PromotionData(Player2.getId(), Player2.data.avatarUrl, Player2.data.name, scores, teammateData.rank, this.rooms.size() == 1));
                objects.add(Player1);
                objects.add(Player2);
                for (int i = 1; i <= room.data.result.size(); i++) {
                    Player tmPlayer = this.actors.getOrDefault(room.data.result.getOrDefault(i, null).playerId, null);
                    if (tmPlayer != null
                            && !tmPlayer.getId().equals(Player1.getId())
                            && !tmPlayer.getId().equals(teammateData.player.getId())) {
                        tmPlayer.data.matchPoints = tmPlayer.data.matchPoints - scores;
                        list.add(new Struct.PromotionData(tmPlayer.getId(), tmPlayer.data.avatarUrl, tmPlayer.data.name, -scores, i, this.rooms.size() == 1));
                        objects.add(tmPlayer);
                    }
                }
            }
            roomMap.put(room.data.roomId, list);
        }
        //排序
        objects.sort((o1, o2) -> Long.compare(o2.data.matchPoints, o1.data.matchPoints));
        for (int i = 1; i <= objects.size(); i++) {
            Player player = objects.get(i - 1);
            player.data.roomId = 0;
            Map<String, Object> messageMap = new HashMap<>();
            String message;
            int cmd;
            //如果只有一张桌子则直接结束
            if (this.rooms.size() <= 1) {
                HashMap<String, Object> map = new HashMap<>();
                JSONArray rewards = this.prop.rewards;
                switch (i) {
                    case 1:
                        processRank(1, player, rewards, map);
                        break;
                    case 2:
                        processRank(2, player, rewards, map);
                        break;
                    case 3:
                        processRank(3, player, rewards, map);
                        break;
                    case 4:
                        processRank(4, player, rewards, map);
                        break;
                }
                cmd = CommonProto.Cmd.FirstPlayer_VALUE;
                map.put("QR", "");
                // 创建一个Date对象，表示当前时间
                Date now = new Date();
                // 定义日期时间格式
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                // 格式化Date对象为字符串
                String formattedDateTime = formatter.format(now);
                map.put("date", formattedDateTime);
                map.put("contestName", this.prop.desc);
                messageMap.put("message", map);
                player.receive(cmd, messageMap);

                cn.hutool.json.JSONObject object = new cn.hutool.json.JSONObject();
                object.put("type", "GetCompeteReward");
                object.put("userId", player.data.playerId);
                object.put("competeId", this.data.contestId);
                object.put("rank", i);
                try {
                    RabbitMQUtil.sendMessage(object.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                if (i <= this.propPromotionLv.num) {
                    roomMap.forEach((key, value) -> {
                        Optional<Struct.PromotionData> first = value.stream()
                                .filter(promotionData -> promotionData.getPlayerId().equals(player.getId()))
                                .findFirst();
                        if (first.isPresent()) {
                            Struct.PromotionData promotionData = first.get();
                            promotionData.setPromotion(true);
                        }
                    });
                    message = "恭喜你晋级下一轮比赛";
                    this.waitingPlayers.add(player.getId());
                    cmd = CommonProto.Cmd.PromotePlayer_VALUE;
                    messageMap.put("message", message);
                    player.receive(cmd, messageMap);
                } else {
                    roomMap.forEach((key, value) -> {
                        Optional<Struct.PromotionData> first = value.stream()
                                .filter(promotionData -> promotionData.getPlayerId().equals(player.getId()))
                                .findFirst();
                        if (first.isPresent()) {
                            Struct.PromotionData promotionData = first.get();
                            promotionData.setPromotion(false);
                        }
                    });
                    message = "很抱歉,您已被淘汰";
                    this.waitingPlayers.remove(player.getId());
                    this.DieOutPlayers.add(player.getId());
                    cmd = CommonProto.Cmd.DieOutPlayer_VALUE;
                    messageMap.put("message", message);
                    player.receive(cmd, messageMap);
                }
            }
        }
        this.data.promotionMap.put(this.data.curRound, roomMap);
    }


    private void processRank(int rank, Player player, JSONArray rewards, Map<String, Object> map) {
        if (rewards != null) {
            StringBuilder msg = new StringBuilder();
            for (Object o : rewards) {
                cn.hutool.json.JSONObject reward = (cn.hutool.json.JSONObject) o;
                String rank1 = (String) reward.get("rank");
                String[] split = rank1.split("-");
                if (split.length > 1) {
                    int min = Integer.parseInt(split[0]);
                    int max = Integer.parseInt(split[1]);
                    if (rank >= min && rank <= max) {
                        getReward(reward, msg);//领奖品处理
                    }
                } else {
                    if (Integer.parseInt(split[0]) == rank) {
                        getReward(reward, msg);//领奖品处理
                    }
                }
            }
            map.put("award", msg.toString());
        }
        map.put("rank", rank);
        map.put("name", player.data.name);
    }

    private static void getReward(cn.hutool.json.JSONObject reward, StringBuilder msg) {
        JSONArray objects = new JSONArray(reward.get("reward"));
        for (Object object : objects) {
            cn.hutool.json.JSONObject obj = (cn.hutool.json.JSONObject) object;
            msg.append(obj.get("goodsName")).append("x").append(obj.get("goodsNum")).append(",");
        }
    }

    /**
     * 比赛结束
     */
    public void gameOver() {
        //清空房间玩家
//        this.rooms.forEach((id, room) -> {
//            room.actors.clear();
//        });
//        this.state = 3;
        this.contestState = 3;
        this.time = 0;
        Log.debug("比赛本轮结束...比赛 : {}, {}, 人数 : {}, 当前轮次 : {}",
                this.data.contestId, this.prop.desc, this.actors.size(), this.data.curRound);
    }

    /**
     * 比赛重置逻辑
     */
    public void resetContest() {

    }

    /**
     * 重置比赛，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁比赛，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存中移除
        ContestService.getInstance().contestMap.remove(this.data.contestId);
        // 重置数据
        this.resetContest();
        // 归还对象
        ContestService.getInstance().contestPool.releaseContest(this);
        Log.debug("Contest {} has been destroyed.", this.data.contestId);
    }
}
